/**
 * BlackBerry Limited proprietary and confidential.
 * Do not reproduce without permission in writing.
 * Copyright (c) 2019 BlackBerry Ltd.
 * All rights reserved.
 */
package com.rim.platform.samples.rest.oidc;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.Security;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.util.EntityUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Sample application requesting a client credentials grant with a private key.
 */
public class SampleWithClientCredsAndPrivateKey extends AbstractRestSample
{
    /**
     * Simple logger. See simplelogger.properties to change the logging settings
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(SampleWithClientCredsAndPrivateKey.class);

    /**
     * Main entry point for the sample.
     * 
     * @param args
     *            the arguments for the sample
     * @throws Exception
     */
    public static void main(String[] args) throws Exception
    {
        // Create the sample application
        SampleWithClientCredsAndPrivateKey sample = new SampleWithClientCredsAndPrivateKey();

        // Create CLI options
        Options options = new Options();
        options.addOption(Option.builder("t").desc("Tenant id").hasArg().required().build());
        options.addOption(Option.builder("c").desc("Client id").hasArg().required().build());
        options.addOption(Option.builder("pk").desc("Private key file").hasArg().required().build());
        options.addOption(Option.builder("eid").desc("EID endpoint - Optional").hasArg().build());
        options.addOption(Option.builder("uem").desc("UEM endpoint").hasArg().required().build());

        try
        {
            // Parse the command line
            CommandLineParser parser = new DefaultParser();
            CommandLine cmd = parser.parse(options, args);

            // Get the values from the command line
            String eidUrl = cmd.getOptionValue("eid", "https://idp.blackberry.com");
            String tenantId = cmd.getOptionValue("t");
            String clientId = cmd.getOptionValue("c");
            String privateKeyFile = cmd.getOptionValue("pk");
            String restApiUrl = cmd.getOptionValue("uem") + "/" + tenantId + "/api/v1/ping";
            String scope = "MDMBWS.All";

            // Log the passed values
            LOGGER.debug("EID URL: {}", eidUrl);
            LOGGER.debug("Tenant Id: {}", tenantId);
            LOGGER.debug("Client Id: {}", clientId);
            LOGGER.debug("Private Key File: {}", privateKeyFile);
            LOGGER.debug("REST API URL: {}", restApiUrl);
            LOGGER.debug("Scope: {}", scope);

            // Get the token endpoint URL for the tenant from the discovery document
            LOGGER.info("Getting the token endpoint URL for the tenant from the discovery document");
            String endpointUrl = sample.getOidcDiscoveryDocumentService().getEndpointUrl(eidUrl,
                    OidcDiscoveryDocumentService.EndpointKey.TOKEN_ENDPOINT, tenantId);

            // Add the BouncyCastle encryption provider
            Security.addProvider(new BouncyCastleProvider());

            // Load the private key
            LOGGER.info("Loading the private key from {}", privateKeyFile);
            byte[] privateKeyBytes = Files.readAllBytes(Paths.get(privateKeyFile));
            PrivateKey privateKey = KeyFactory.getInstance("RSA")
                    .generatePrivate(new PKCS8EncodedKeySpec(Base64.getDecoder().decode(privateKeyBytes)));

            // Request an access token
            LOGGER.info("Requesting an access token");
            String accessToken = sample.getAccessToken(endpointUrl, clientId, privateKey, null, scope);
            LOGGER.debug("Access token: {}", accessToken);

            // Invoke the REST API with the access token
            LOGGER.info("Invoking the REST API with the access token");
            try (CloseableHttpResponse response = sample.invokeRestApi(restApiUrl, accessToken))
            {
                LOGGER.info("Response code: {}", response.getStatusLine().getStatusCode());
                LOGGER.info("Content: {}", EntityUtils.toString(response.getEntity()));
            }
        }
        catch (ParseException e)
        {
            LOGGER.error(e.getMessage(), e);
            HelpFormatter formatter = new HelpFormatter();
            formatter.printHelp("sample", options);
        }
    }
}
